class SvgConstraintElement extends SvgMapElement {
    constructor(source) {
        super();
        this.textOffsetRatio = 3;
        this.showText = true;
        this.minimize = false;
        this._textWidth = NaN;
        this._textHeight = NaN;
        this.needRepaint = false;
        this._lastX = 0;
        this._lastY = 0;
        this._lastMinimize = false;
        this.source = source;
        this._id = SvgConstraintElement._ID++;
    }
    get coordinates() {
        if (this._coordinates) {
            return this._coordinates;
        }
        if (this.source && this.source.infos.coordinates) {
            return this.source.infos.coordinates;
        }
    }
    set coordinates(v) {
        this._coordinates = v;
    }
    id(map) {
        return "constraint-" + this.source.icaoNoSpace + "-map-" + map.index;
        ;
    }
    imageFileName() {
        return "ICON_CSTR_MAGENTA.svg";
    }
    createDraw(map) {
        let fontSize = map.config.waypointLabelFontSize / map.overdrawFactor;
        let text = "CSTR";
        let speedText = "";
        if (this.source) {
            text = (this.source.legAltitude1 / 10).toFixed(0) + "0";
        }
        if (this.source.speedConstraint > 10) {
            speedText = this.source.speedConstraint.toFixed(0) + "KT";
        }
        let c = document.createElement("canvas");
        let ctx = c.getContext("2d");
        ctx.font = fontSize + "px " + map.config.waypointLabelFontFamily;
        this._textWidth = Math.max(ctx.measureText(text).width, ctx.measureText(speedText).width);
        this._textHeight = fontSize * 0.675;
        this._label = undefined;
        this.needRepaint = true;
        let labelId = this.id(map) + "-text-" + map.index;
        let label = document.getElementById(labelId);
        if (label instanceof SVGForeignObjectElement) {
            this._label = label;
            this.needRepaint = true;
        }
        if (!this._label) {
            this._label = document.createElementNS("http://www.w3.org/2000/svg", "foreignObject");
            this._label.id = labelId;
            this._label.setAttribute("width", (this._textWidth + map.config.waypointLabelBackgroundPaddingLeft / map.overdrawFactor + map.config.waypointLabelBackgroundPaddingRight).toFixed(0) + "px");
            this._label.setAttribute("height", (this._textHeight * (this.source.speedConstraint > 0 ? 2.2 : 1) + map.config.waypointLabelBackgroundPaddingTop + map.config.waypointLabelBackgroundPaddingBottom).toFixed(0) + "px");
            let canvas = document.createElement("canvas");
            canvas.setAttribute("width", (this._textWidth + map.config.waypointLabelBackgroundPaddingLeft / map.overdrawFactor + map.config.waypointLabelBackgroundPaddingRight).toFixed(0) + "px");
            canvas.setAttribute("height", (this._textHeight * (this.source.speedConstraint > 0 ? 2.2 : 1) + map.config.waypointLabelBackgroundPaddingTop + map.config.waypointLabelBackgroundPaddingBottom).toFixed(0) + "px");
            this._label.appendChild(canvas);
            let context = canvas.getContext("2d");
            if (map.config.waypointLabelUseBackground) {
                context.fillStyle = "black";
                context.fillRect(0, 0, this._textWidth + map.config.waypointLabelBackgroundPaddingLeft / map.overdrawFactor + map.config.waypointLabelBackgroundPaddingRight, this._textHeight + map.config.waypointLabelBackgroundPaddingTop + map.config.waypointLabelBackgroundPaddingBottom);
            }
            let activeWaypoint = FlightPlanManager.DEBUG_INSTANCE.getActiveWaypoint(false, true);
            if (this.source.ident === activeWaypoint.ident) {
                context.fillStyle = "#D570FF";
            }
            else {
                context.fillStyle = "white";
            }
            context.font = fontSize + "px " + map.config.waypointLabelFontFamily;
            context.fillText(text, map.config.waypointLabelBackgroundPaddingLeft, this._textHeight + map.config.waypointLabelBackgroundPaddingTop - 22);
            map.textLayer.appendChild(this._label);
        }
        this.svgElement = document.createElementNS(Avionics.SVG.NS, "image");
        this.svgElement.id = this.id(map);
        this.svgElement.classList.add("constraint-icon");
        this.svgElement.setAttribute("hasTextBox", "true");
        this.svgElement.setAttribute("width", fastToFixed(map.config.waypointIconSize / map.overdrawFactor * 1.3, 0));
        this.svgElement.setAttribute("height", fastToFixed(map.config.waypointIconSize / map.overdrawFactor * 1.3, 0));
        return this.svgElement;
    }
    updateDraw(map) {
        if (this.coordinates) {
            map.coordinatesToXYToRef(this.coordinates, this);
        }
        if (isFinite(this.x) && isFinite(this.y)) {
            if (this.svgElement && this._lastMinimize !== this.minimize) {
                if (this.minimize) {
                    this.svgElement.setAttribute("width", fastToFixed(map.config.waypointIconSize / map.overdrawFactor * 1.3 * 0.5, 0));
                    this.svgElement.setAttribute("height", fastToFixed(map.config.waypointIconSize / map.overdrawFactor * 1.3 * 0.5, 0));
                }
                else {
                    this.svgElement.setAttribute("width", fastToFixed(map.config.waypointIconSize / map.overdrawFactor * 1.3, 0));
                    this.svgElement.setAttribute("height", fastToFixed(map.config.waypointIconSize / map.overdrawFactor * 1.3, 0));
                }
                this._lastMinimize = this.minimize;
                this.needRepaint = true;
            }
            if (this.needRepaint || Math.abs(this._lastX - this.x) > 0.1 || Math.abs(this._lastY - this.y) > 0.1) {
                this._lastX = this.x;
                this._lastY = this.y;
                let x = (this.x - map.config.waypointIconSize / map.overdrawFactor * 1.3 * 0.5 * (this.minimize ? 0.5 : 1));
                let y = (this.y - map.config.waypointIconSize / map.overdrawFactor * 1.3 * 0.5 * (this.minimize ? 0.5 : 1));
                this.svgElement.setAttribute("x", x + "");
                this.svgElement.setAttribute("y", y + "");
                if (!this._label) {
                    let labelId = this.id(map) + "-text-" + map.index;
                    let label = document.getElementById(labelId);
                    if (label instanceof SVGForeignObjectElement) {
                        let c = document.createElement("canvas");
                        let ctx = c.getContext("2d");
                        let fontSize = map.config.waypointLabelFontSize / map.overdrawFactor;
                        let text = "CSTR";
                        ctx.font = fontSize + "px " + map.config.waypointLabelFontFamily;
                        this._textWidth = ctx.measureText(text).width;
                        this._textHeight = fontSize * 0.675;
                        this._label = label;
                        this.needRepaint = true;
                    }
                }
                if (this._label) {
                    let lastWaypoint = FlightPlanManager.DEBUG_INSTANCE.getPreviousActiveWaypoint();
                    if (this.source.ident === lastWaypoint.ident) {
                        this._label.style.visibility = "hidden";
                    }
                    let activeWaypoint = FlightPlanManager.DEBUG_INSTANCE.getActiveWaypoint(false, true);
                    if (this.source.ident === activeWaypoint.ident) {
                        this._label.setAttribute("stroke", "#D570FF");
                    }
                    let textX = (x + map.config.waypointIconSize * 0.5 - this._textWidth * 0.5 + map.config.waypointLabelDistanceX / map.overdrawFactor) - 2;
                    let textY = y + map.config.waypointLabelDistance + map.config.waypointLabelFontSize / map.overdrawFactor + 4;
                    this._label.setAttribute("x", textX + "");
                    this._label.setAttribute("y", textY + "");
                    this.needRepaint = false;
                }
                else {
                    this.needRepaint = true;
                }
            }
        }
    }
}
SvgConstraintElement._ID = 0;
class SvgTopOfXElement extends SvgMapElement {
    constructor(name, imageName) {
        super();
        this.imageName = imageName;
        this.heading = 0;
        this._lastX = 0;
        this._lastY = 0;
        this._lastFileName = "";
        this.name = name;
        if (!this.name) {
            this.name = (Math.random() * 10).toFixed(0).padStart(10, "0");
        }
    }
    id(map) {
        return this.name + "-map-" + map.index;
        ;
    }
    imageFileName() {
        return this.imageName + ".svg";
    }
    createDraw(map) {
        let container = document.createElementNS(Avionics.SVG.NS, "image");
        container.id = this.id(map);
        container.classList.add("map-city-icon");
        container.setAttribute("width", fastToFixed(map.config.waypointIconSize / map.overdrawFactor * 2, 0));
        container.setAttribute("height", fastToFixed(map.config.waypointIconSize / map.overdrawFactor * 2, 0));
        container.setAttributeNS("http://www.w3.org/1999/xlink", "href", map.config.imagesDir + this.imageFileName());
        return container;
    }
    updateDraw(map) {
        map.latLongToXYToRef(this.lat, this.long, this);
        if (isFinite(this.x) && isFinite(this.y)) {
            let fileName = this.imageFileName();
            if (fileName !== this._lastFileName) {
                this.svgElement.setAttributeNS("http://www.w3.org/1999/xlink", "href", map.config.imagesDir + fileName);
                this._lastFileName = fileName;
            }
            if (Math.abs(this.x - this._lastX) > 0.1 || Math.abs(this.y - this._lastY) > 0.1) {
                this.svgElement.setAttribute("x", fastToFixed((this.x - map.config.waypointIconSize / map.overdrawFactor), 1));
                this.svgElement.setAttribute("y", fastToFixed((this.y - map.config.waypointIconSize / map.overdrawFactor), 1));
                let angle = this.heading;
                if (map.rotationMode != EMapRotationMode.NorthUp) {
                    angle -= map.mapUpDirection;
                }
                this.svgElement.setAttribute("transform", "rotate(" + angle.toFixed(0) + " " + this.x.toFixed(0) + " " + this.y.toFixed(0) + ")");
                this._lastX = this.x;
                this._lastY = this.y;
            }
        }
    }
}
//# sourceMappingURL=SvgConstraintElement.js.map